package com.diss.omppapp.services;

import com.diss.omppapp.entities.SubmissionEntity;
import com.diss.omppapp.repositories.SubmissionEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.persistence.criteria.CriteriaBuilder;
import java.sql.Date;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@Component
public class SubmissionService {

    @Autowired
    private SubmissionEntityRepository submissionEntityRepository;

    public SubmissionService(){
    }

    public List<SubmissionEntity> getClassSubmissions(String username, String classCode) {
        Optional<List<SubmissionEntity>> possibleList = submissionEntityRepository.findByUsernameAndClassCodeAndSubmitted(username,classCode, true);
        return possibleList.orElseGet(ArrayList::new);
    }

    public List<SubmissionEntity> getPaperSubmissions(String paperId) {
        int id = -1;
        try {
            id = Integer.parseInt(paperId);
        } catch (NumberFormatException nfe) {
            return new ArrayList<>();
        }

        Optional<List<SubmissionEntity>> possibleList = submissionEntityRepository.findByPaperIdAndSubmitted(id, true);
        return possibleList.orElseGet(ArrayList::new);
    }

    public Boolean createSubmission(Map<String,String> entity) {
        SubmissionEntity submission;
        try {
            submission = createEntity(entity);
        } catch (IllegalArgumentException iae) {
            return false;
        }

        if(entity.get("isSubmitted").equals("true")) {
            submission.setSubmitted(true);
        } else {
            return false;
        }

        submissionEntityRepository.save(submission);
        return true;
    }

    public Boolean createDraft(Map<String,String> entity) {
        SubmissionEntity submission;
        try {
            submission = createEntity(entity);
        } catch (IllegalArgumentException iae) {
            return false;
        }

        if(entity.get("isSubmitted").equals("false")) {
            submission.setSubmitted(false);
        } else {
            return false;
        }

        submissionEntityRepository.save(submission);
        return true;
    }

    private SubmissionEntity createEntity(Map<String, String> entityMap) throws IllegalArgumentException {
        SubmissionEntity submission = new SubmissionEntity();
        int paperId = Integer.parseInt(entityMap.get("paperId"));
        submission.setPaperId(paperId);
        submission.setUsername(entityMap.get("username"));
        submission.setClassCode(entityMap.get("classCode"));
        submission.setSubmissionText(entityMap.get("submissionText"));
        submission.setSubmitted(false);
        submission.setSubmissionDate(Date.valueOf(entityMap.get("submissionDate")));
        return submission;
    }

}
